#include <gpib.h>
#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "generator.h"

#define GEN 8

static int panelHandle;
                                                   
short IAdress[2]={GEN,-1};                        
char QStr[255], data[10000];
float Freq=10000, Alpha=0, Um=4.5, GraphData[360];
short delka;

union DTyp
{ 
  char a[20000];
  unsigned short b[10000];
} Data;  
  
                                                   
void chyby(char msg[30])                           
{                                                  
#ifdef DEBUG
  if (ibsta >= 0x08000) {                          
    sprintf(data, "%s\nibsta = %xH\niberr = %d\n", msg, ibsta, iberr);
    
    MessagePopup("Error", data);
    EnableLocal(0, IAdress);                       
    exit(EXIT_FAILURE);                            
  } 
#endif  
}                                                  

void Query(short Adr, char QString[255])                
{                                                       
  Send(0, Adr, QString, strlen(QString), 2);              
  chyby ("Send status: ");                              
  Receive(0, Adr, data, 255, 256);                      
  chyby("Receive status: ");                            
  data[ibcnt]=0;                                        
}                                                       

void SendCmd(short Adr, char QString[255])                
{                                                       
  Send(0, Adr, QString, strlen(QString), 2);              
  chyby ("Send status: ");                              
}                                                       

void SendBlok(short Adr, char QString[10300], short Delka)                
{                                                       
  Send(0, Adr, QString, Delka, 2);              
  chyby ("Send status: ");                              
}                                                       
                                                   

void idn(short Adr)                                     
{                                                       
  Query(Adr, "*idn?");                                  
  data[ibcnt-1]=0;
  MessagePopup("IDN", data);
}                                                       
       

void GenSin(float A, float U)
{
  short i;
  
  /*Generovani dat, ktera se posilaji do generatoru. Tzn. 3600 bodu, 
  max. hodnota 2047 (protoze v gen. je 12bitovy prevodnik)*/
  for (i=0; i<=10*A; i++) Data.b[i]= 0;
  for (i++; i<1800; i++) Data.b[i] = sin(1.745e-3*(float)i)*2047;
  for (i++; i<=10*A+1800; i++) Data.b[i]= 0;
  for (i++; i<3600; i++) Data.b[i] = sin(1.745e-3*(float)i)*2047;
}
                                                        

void SendSin(float F, float A, float U)
{
  short i;

  //rozsviti LED
  SetCtrlVal(PANEL, PANEL_LED,1);
  
  //nastavi pozadovanou frekvenci
  sprintf(data,"freq %f", F);
  SendCmd(GEN, data);
  
  //nastavi pozadovanou amplitudu
  sprintf(data,"volt %f", U);
  SendCmd(GEN, data);

  //generovani prikazu pro blok. prenos
  sprintf(data,"data:dac volatile,#47200");
  GenSin(Alpha, Um);  
  delka=strlen(data);
  for (i=0; i<7200; i++) {
    data[delka+i]=Data.a[i];
  }  
  
  //prekresleni prubehu v grafu
  DeleteGraphPlot(PANEL, PANEL_GRAPH, -1, VAL_IMMEDIATE_DRAW);
  PlotWaveform (PANEL, PANEL_GRAPH, &Data, 3600, VAL_FLOAT, 4.8828e-4,
              0, 0, 0.1, VAL_THIN_LINE, VAL_CONNECTED_POINTS,
              VAL_SOLID, VAL_CONNECTED_POINTS, VAL_RED);
  
  //odeslani odeslani bloku dat a vyvolani pozad. prubehu
  SendBlok(GEN,data,delka+7200);
  SendCmd(GEN,"FORM:BORD SWAP");  
  SendCmd(GEN,"FUNC:USER VOLATILE");
  SendCmd(GEN,"FUNC:SHAP USER");
  SetCtrlVal(PANEL, PANEL_LED,0);
}

int main (int argc, char *argv[])
{
	SendIFC(0);
	EnableRemote(0, IAdress);
	
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "generator.uir", PANEL)) < 0)
		return -1;
	//idn(GEN);
	SendSin(Freq, Alpha, Um);

	DisplayPanel (panelHandle);
	RunUserInterface ();
	DiscardPanel (panelHandle);
	return 0;
}

int CVICALLBACK Exit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			EnableLocal(0, IAdress);
			break;
		}
	return 0;
}

int CVICALLBACK Ampl (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(panel, PANEL_NUMERICAmpl, &Um);
			SendSin(Freq, Alpha, Um);

			break;
		}
	return 0;
}

int CVICALLBACK Frekv (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char rd, i;
	float fr;
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(panel, PANEL_RING, &rd);
			GetCtrlVal(panel, PANEL_NUMERICFrekv, &fr);
			if (rd==6 && fr>1)
			  {
				Freq = 1e6;
				SetCtrlVal(panel, PANEL_NUMERICFrekv, 1);
				SetCtrlVal(panel, PANEL_RING, 6);
			  }
			else for (i=0; i<rd; i++) fr=fr*10;
			Freq=(int)fr;
			SendSin(Freq, Alpha, Um);
			break;
		}
	return 0;
}

int CVICALLBACK Uhel (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(panel, PANEL_NUMERICUhel, &Alpha);
			SendSin(Freq, Alpha, Um);

			break;
		}
	return 0;
}

int CVICALLBACK rad (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{   char rd, i;
	float fr;
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(panel, PANEL_RING, &rd);
			GetCtrlVal(panel, PANEL_NUMERICFrekv, &fr);
			if (rd==6 && fr>1)
			  {
				Freq = 1e6;
				SetCtrlVal(panel, PANEL_NUMERICFrekv, 1);
				SetCtrlVal(panel, PANEL_RING, 6);
			  }
			else for (i=0; i<rd; i++) fr=fr*10;
			Freq=(int)fr;
			SendSin(Freq, Alpha, Um);
			break;
		}
	return 0;
}

int CVICALLBACK Graf (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:

			break;
		}
	return 0;
}
